from frasco.mail.provider import MailProvider
import requests
import logging


class MailgunProvider(MailProvider):
    def send(self, msg):
        r = requests.post("https://api.mailgun.net/v3/%s/messages.mime" % self.options['mailgun_domain'],
            auth=("api", self.options['mailgun_api_key']), data={'to': msg.recipients}, files={'message': msg.as_string()})
        r.raise_for_status()



from flask import request, abort, current_app, has_request_context
from .utils import unknown_value
from .ctx import FlagContextStack
from werkzeug.exceptions import HTTPException
from dateutil.parser import parse as parse_date
import functools
import inspect


try:
    from marshmallow import Schema as MarshmallowSchema
    from marshmallow.exceptions import ValidationError as MarshmallowValidationError
    marshmallow_available = True
except ImportError:
    marshmallow_available = False


def _value_from_multidict(v, aslist=False):
    if aslist:
        return v
    if len(v) == 0:
        return None
    if len(v) == 1:
        return v[0]
    return v


class MissingRequestParam(Exception):
    pass


class RequestParamCoerceError(Exception):
    pass


def get_request_param_value(name, location=None, aslist=False, request_data=None):
    if request_data is not None:
        if name not in request_data:
            raise MissingRequestParam()
        value = request_data[name]
        if aslist and not isinstance(value, (tuple, list)):
            value = [] if value is None else [value]
        return value

    if (not location or location == 'view_args') and name in request.view_args:
        if aslist and not isinstance(request.view_args[name], list):
            return [] if request.view_args[name] is None else [request.view_args[name]]
        return request.view_args[name]

    if not request.is_json:
        if not location or location == 'values':
            if name in request.values:
                return _value_from_multidict(request.values.getlist(name), aslist)
        elif location == 'args' and name in request.args:
            return _value_from_multidict(request.args.getlist(name), aslist)
        elif location == 'form' and name in request.form:
            return _value_from_multidict(request.form.getlist(name), aslist)

    if (not location or location == 'json') and request.is_json:
        data = request.get_json(silent=True)
        if isinstance(data, dict) and name in data:
            if aslist and not isinstance(data[name], list):
                return [] if data[name] is None else [data[name]]
            return data[name]

    if (not location or location == 'files') and name in request.files:
        return _value_from_multidict(request.files.getlist(name), aslist)

    raise MissingRequestParam()


class RequestParam(object):
    def __init__(self, name, type=None, nullable=False, required=False, loader=None, validator=None,
                 dest=None, location=None, aslist=False, aslist_iter_coerce=True, aslist_iter_loader=True,
                 help=None, default=unknown_value, **loader_kwargs):
        self.name = name
        self.type = type
        self.nullable = nullable
        self.required = required
        self.loader = loader
        self.loader_kwargs = loader_kwargs
        self.validator = validator
        self.dest = dest
        self.location = location
        self.aslist = aslist
        self.aslist_iter_coerce = aslist_iter_coerce
        self.aslist_iter_loader = aslist_iter_loader
        self.help = help
        self.default = default

    @property
    def names(self):
        return self.name if isinstance(self.name, tuple) else (self.name,)

    @property
    def types(self):
        if not self.type:
            return
        return self.type if isinstance(self.type, tuple) else (self.type,)

    @property
    def names_types(self):
        names = self.names
        fallback_type = None
        types = []
        if self.type:
            types = self.types
            fallback_type = types[-1]
        return [(names[i], types[i] if i < len(types) else fallback_type) for i in range(len(names))]

    @property
    def dests(self):
        if self.dest is None:
            return self.names
        elif self.dest and not isinstance(self.dest, tuple):
            return (self.dest,)
        return self.dest

    def _abort(self, message, code=400):
        abort(code, u"%s: %s" % (", ".join(self.names), message))

    def process(self, request_data=None):
        try:
            values = self.extract(request_data)
        except MissingRequestParam:
            if self.required and self.default is unknown_value:
                if self.required is True or self.required(request_data):
                    self._abort("required field")
            if not self.default is unknown_value:
                if not isinstance(self.name, tuple):
                    values = [self.default]
                else:
                    values = self.default
            else:
                return dict()

        values = self.load(*values)
        if not self.validate(*values):
            self._abort("invalid field")
        if self.dests is False:
            return {}
        if len(self.dests) != len(values):
            raise Exception('Mismatch between length of dest and number of values')
        return dict(zip(self.dests, values))

    def extract(self, request_data=None):
        values = []
        for name, type in self.names_types:
            value = get_request_param_value(name, self.location, self.aslist, request_data)
            if value is None and not self.nullable:
                raise MissingRequestParam()
            if type and value is not None:
                if self.aslist and self.aslist_iter_coerce:
                    value = map(lambda v: self.coerce(v, type), value)
                else:
                    value = self.coerce(value, type)
            values.append(value)
        return values

    def coerce(self, value, type):
        if marshmallow_available:
            try:
                schema = None
                if inspect.isclass(type) and issubclass(type, MarshmallowSchema):
                    schema = type()
                elif isinstance(type, MarshmallowSchema):
                    schema = type
                if schema:
                    return schema.load(value)
            except MarshmallowValidationError as e:
                self._abort("\n - " + "\n - ".join(u"%s: %s" % i for i in e.normalized_messages().items()))

        try:
            if type is bool:
                if isinstance(value, (str, unicode)):
                    return value.lower() not in ('False', 'false', 'no', '0')
                return bool(value)
            return type(value)
        except HTTPException as e:
            self._abort(e.description, e.code)
        except:
            self._abort("invalid value")

    def load(self, *values):
        if self.loader:
            try:
                if self.aslist and self.aslist_iter_loader:
                    rv = []
                    for t in zip(*values):
                        rv.append(self.loader(*t, **self.loader_kwargs))
                    return (rv,)
                else:
                    rv = self.loader(*values, **self.loader_kwargs)
                    if not isinstance(rv, tuple):
                        return (rv,)
                    return rv
            except HTTPException as e:
                self._abort(e.description, e.code)
        return values

    def validate(self, *values):
        return not self.validator or self.validator(*values)

    def update_kwargs(self, kwargs, request_data=None):
        for name in self.names:
            kwargs.pop(name, None)
        kwargs.update(self.process(request_data))
        return kwargs


disable_request_params = FlagContextStack()


def update_kwargs_with_request_params(kwargs, request_params, request_data=None):
    for param in request_params:
        param.update_kwargs(kwargs, request_data)
    return kwargs


def wrap_request_param_func(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        if has_request_context() and not disable_request_params.top:
            update_kwargs_with_request_params(kwargs, wrapper.request_params)
        return func(*args, **kwargs)
    wrapper.request_params = []
    return wrapper


def request_param(name, type=None, cls=RequestParam, append=True, **kwargs):
    def decorator(func):
        if not append or not hasattr(func, 'request_params'):
            wrapper = wrap_request_param_func(func)
        else:
            wrapper = func
        if isinstance(name, cls):
            wrapper.request_params.append(name)
        else:
            wrapper.request_params.append(cls(name, type, **kwargs))
        return wrapper
    return decorator


def partial_request_param(name=None, type=None, cls=RequestParam, **kwargs):
    def decorator(name_=None, **kw):
        return request_param(name_ or name, type, cls=cls, **dict(kwargs, **kw))
    decorator.request_param = cls(name, type, **kwargs)
    return decorator


def partial_request_param_loader(name=None, type=None, **kwargs):
    def decorator(loader_func):
        return partial_request_param(name, type, loader=loader_func, **kwargs)
    return decorator


def _dict_to_request_params(params_dict, cls=RequestParam):
    params = []
    for name, kwargs in params_dict.iteritems():
        if isinstance(kwargs, cls):
            params.append(kwargs)
            continue
        if not isinstance(kwargs, dict):
            kwargs = dict(type=kwargs)
        params.append(cls(name, **kwargs))
    return params


def request_params(params, cls=RequestParam):
    def decorator(func):
        for param in (_dict_to_request_params(params, cls) if isinstance(params, dict) else params):
            func = request_param(param, cls=cls)(func)
        return func
    decorator.params = params
    return decorator


def nested(params_dict, cls=RequestParam, **kwargs):
    params = _dict_to_request_params(params_dict, cls)
    def coerce(value):
        if not isinstance(value, dict):
            abort(400, "not a hash")
        out = {}
        for param in params:
            param.update_kwargs(out, value)
        return out
    return dict(type=coerce, **kwargs) if kwargs else coerce


def list_of(type, **kwargs):
    coerce_obj = nested(type) if isinstance(type, dict) else type
    def coerce(value):
        if not isinstance(value, (list, tuple)):
            abort(400, "not a list")
        return [coerce_obj(item) for item in value]
    kwargs['aslist'] = True
    kwargs['aslist_iter_coerce'] = False
    return dict(type=coerce, **kwargs)


def utcdate(**kwargs):
    def wrapper(value):
        if not value:
            return
        dt = parse_date(value, ignoretz=True, **kwargs)
        return dt.date()
    return wrapper


def utcdatetime(**kwargs):
    def wrapper(value):
        if not value:
            return
        return parse_date(value, **kwargs)
    return wrapper


def required_if_other(other_params):
    if not isinstance(other_params, dict):
        other_params = dict([(other_params, unknown_value)])
    def required(request_data):
        for param, expected_value in other_params.items():
            try:
                value = get_request_param_value(param, request_data=request_data)
                if callable(expected_value):
                    return expected_value(value)
                return expected_value is unknown_value or value == expected_value
            except MissingRequestParam:
                pass
        return False
    return required


def one_of(*items):
    def validator(value):
        return value in items
    return validator



from werkzeug.utils import import_string as wz_import_string, cached_property
from werkzeug.local import LocalProxy, LocalStack
from flask import Markup, json, request, _request_ctx_stack, has_request_context, abort, current_app
import imp
import functools
import re
from slugify import slugify
import yaml
import speaklater
import os
from contextlib import contextmanager
import subprocess
import click
import inspect
import logging
from .helpers import url_for


def import_string(impstr, attr=None):
    """Imports a string. Can import an attribute of the imported
    class/module using a double colon as a separator
    """
    if inspect.isclass(impstr):
        return impstr
    if "::" in impstr:
        impstr, attr = impstr.split("::")
    imported = wz_import_string(impstr)
    if attr is not None:
        return getobjpath(imported, attr)
    return imported


def getobjpath(obj, path):
    """Returns an item or attribute of the object recursively.
    Item names are specified between brackets, eg: [item].
    Attribute names are prefixed with a dot (the first one is optional), eg: .attr
    Example: getobjpath(obj, "attr1.attr2[item].attr3")
    """
    if not path:
        return obj
    if path.startswith("["):
        item = path[1:path.index("]")]
        return getobjpath(obj[item], path[len(item) + 2:])
    if path.startswith("."):
        path = path[1:]
    if "." in path or "[" in path:
        dot_idx = path.find(".")
        bracket_idx = path.find("[")
        if dot_idx == -1 or bracket_idx < dot_idx:
            idx = bracket_idx
            next_idx = idx
        else:
            idx = dot_idx
            next_idx = idx + 1
        attr = path[:idx]
        return getobjpath(getattr(obj, attr), path[next_idx:])
    return getattr(obj, path)


def find_classes_in_module(module, clstypes):
    """Find classes of clstypes in module
    """
    classes = []
    for item in dir(module):
        item = getattr(module, item)
        try:
            for cls in clstypes:
                if issubclass(item, cls) and item != cls:
                    classes.append(item)
        except:
            pass
    return classes


class ImportClassError(ImportError):
    pass


def find_class_in_module(cls, clstypes):
    if not isinstance(clstypes, (tuple, list)):
        clstypes = (clstypes,)

    if inspect.ismodule(cls):
        classes = find_classes_in_module(cls, clstypes)
        if not classes:
            raise ImportClassError('No extension class found in module')
        if len(classes) > 1:
            raise ImportClassError('Too many extension classes in module')
        return classes[0]

    if not isinstance(cls, clstypes):
        raise ImportClassError("Wrong class type")
    return cls


def import_class(impstr, clstypes, fallback_package=None):
    try:
        return find_class_in_module(import_string(impstr), clstypes)
    except ImportError as e:
        if not fallback_package or ('No module named' not in str(e) and str(e) != 'No extension class found in module'):
            raise
        return find_class_in_module(import_string(fallback_package + "." + impstr), clstypes)



def remove_yaml_frontmatter(source, return_frontmatter=False):
    """If there's one, remove the YAML front-matter from the source
    """
    if source.startswith("---\n"):
        frontmatter_end = source.find("\n---\n", 4)
        if frontmatter_end == -1:
            frontmatter = source
            source = ""
        else:
            frontmatter = source[0:frontmatter_end]
            source = source[frontmatter_end + 5:]
        if return_frontmatter:
            return (source, frontmatter)
        return source
    if return_frontmatter:
        return (source, None)
    return source


def parse_yaml_frontmatter(source):
    source, frontmatter = remove_yaml_frontmatter(source, True)
    if frontmatter:
        return (yaml.safe_load(frontmatter), source)
    return (None, source)


def populate_obj(obj, attrs):
    """Populates an object's attributes using the provided dict
    """
    for k, v in attrs.iteritems():
        setattr(obj, k, v)


def url_for_static(filename, **kwargs):
    """Shortcut function for url_for('static', filename=filename)
    """
    return url_for('static', filename=filename, **kwargs)


def url_for_same(**overrides):
    return url_for(request.endpoint, **dict(dict(request.args,
        **request.view_args), **overrides))


def wrap_in_markup(f):
    @functools.wraps(f)
    def wrapper(*args, **kwargs):
        return Markup(f(*args, **kwargs))
    return wrapper


def make_kwarg_validator(name, validator_func):
    if not isinstance(name, tuple):
        name = (name,)
    def decorator_gen(**kwargs):
        def decorator(func):
            @functools.wraps(func)
            def wrapper(*args, **kw):
                values = {n: kw.get(n) for n in name}
                if not validator_func(**dict(kwargs, **values)):
                    abort(400)
                return func(*args, **kw)
            return wrapper
        return decorator
    return decorator_gen


def kwarg_validator(name):
    def decorator(validator_func):
        return make_kwarg_validator(name, validator_func)
    return decorator


def inject_app_config(app, config, prefix=None):
    for k, v in config.iteritems():
        if prefix:
            k = "%s%s" % (prefix, k)
        app.config[k.upper()] = v


def extract_unmatched_items(items, allowed_keys, prefix=None, uppercase=False):
    unmatched = {}
    for k, v in items.iteritems():
        if k not in allowed_keys:
            if prefix:
                k = "%s%s" % (prefix, k)
            if uppercase:
                k = k.upper()
            unmatched[k] = v
    return unmatched


def deep_update_dict(a, b):
    for k, v in b.iteritems():
        if k not in a:
            a[k] = v
        elif isinstance(a[k], dict) and isinstance(v, dict):
            deep_update_dict(a[k], v)
        elif isinstance(a[k], list) and isinstance(v, list):
            a[k].extend(v)
        elif isinstance(v, list) and not isinstance(a[k], list):
            a[k] = [a[k]] + v
        else:
            a[k] = v
    return a


class UnknownValue(object):
    pass

unknown_value = UnknownValue()


class AttrDict(dict):
    """Dict which keys are accessible as attributes
    """
    def __getattr__(self, name):
        return self[name]

    def __setattr__(self, name, value):
        self[name] = value

    def __delattr__(self, name):
        del self[name]

    def get_or_raise(self, name, message):
        try:
            return self[name]
        except KeyError:
            raise KeyError(message)

    def for_json(self):
        return dict(self)


class JSONEncoder(json.JSONEncoder):
    """A JSONEncoder which always activates the for_json feature
    """
    def __init__(self, *args, **kwargs):
        kwargs["for_json"] = True
        super(JSONEncoder, self).__init__(*args, **kwargs)

    def default(self, o):
        if isinstance(o, speaklater._LazyString):
            return o.value
        if isinstance(o, set):
            return list(o)
        return json.JSONEncoder.default(self, o)


class ShellError(Exception):
    def __init__(self, returncode, stderr):
        super(ShellError, self).__init__(stderr)
        self.returncode = returncode
        self.stderr = stderr

    def __unicode__(self):
        return unicode(self.stderr)


def shell_exec(args, echo=True, fg="green", **kwargs):
    kwargs["stdout"] = subprocess.PIPE
    kwargs["stderr"] = subprocess.STDOUT
    p = subprocess.Popen(args, **kwargs)
    out, _ = p.communicate()
    if p.returncode > 0:
        raise ShellError(p.returncode, out)
    if echo:
        click.secho(out, fg=fg)
    return out


def get_remote_addr():
    if current_app.debug and "__remoteaddr" in request.values:
        return request.values["__remoteaddr"]
    return request.remote_addr



def match_domains(domain, allowed_domains):
    for allowed_domain in allowed_domains:
        if allowed_domain.startswith('^'):
            if re.search(allowed_domain, domain, re.I):
                return True
        elif allowed_domain.lower() == domain.lower():
            return True
    return False


def match_email_domain(email, allowed_domains):
    if not email or not '@' in email:
        return False
    _, domain = email.split('@')
    return match_domains(domain, allowed_domains)


def remove_all_handlers_from_logger(logger):
    for handler in logger.handlers:
        logger.removeHandler(handler)
    return logger


class ExcludeWerkzeugLogFilter(logging.Filter):
    def filter(self, record):
        return record.name != 'werkzeug'


def join_url_rule(rule1, rule2):
    if not rule1:
        return rule2
    if not rule2:
        return rule1
    return (rule1.rstrip('/') + '/' + rule2.lstrip('/')).rstrip('/')



from flask import current_app, url_for as flask_url_for


def url_for(*args, **kwargs):
    if kwargs.get('_external') and current_app.config.get('FORCE_URL_SCHEME'):
        kwargs.setdefault('_scheme', current_app.config['FORCE_URL_SCHEME'])
    return flask_url_for(*args, **kwargs)



from flask import json
from frasco.ext import get_extension_state
from .ext import redis as current_app_redis


__all__ = ('PartialObject', 'RedisHash', 'JSONRedisHash', 'RedisList', 'JSONRedisList', 'RedisSet', 'JSONRedisSet')


class PartialObject(object):
    def __init__(self, loader, cached_attrs=None):
        object.__setattr__(self, '_loader', loader)
        object.__setattr__(self, "_obj", None)
        object.__setattr__(self, "_cached_attrs", dict(cached_attrs or {}))

    def _load(self):
        if not self._obj:
            object.__setattr__(self, "_obj", self.loader())
        return self._obj

    def __getattr__(self, name):
        if name in self._cached_attrs:
            return self._cached_attrs[name]
        return getattr(self._load(), name)

    def __setattr__(self, name, value):
        if name in self._cached_attrs:
            del self._cached_attrs[name]
        setattr(self._load(), name, value)
        

class RedisObject(object):
    def __init__(self, key, serializer=None, coerce=None, redis=None):
        self.key = key
        self.serializer = serializer
        self.coerce = coerce
        self.redis = redis or current_app_redis

    def _to_redis(self, value):
        if self.serializer:
            return self.serializer.dumps(value)
        return value

    def _from_redis(self, value):
        if value is None:
            return value
        if self.serializer:
            value = self.serializer.loads(value)
        if self.coerce:
            value = self.coerce(value)
        return value

    def clear(self):
        self.redis.delete(self.key)

    def expire(self, ttl):
        self.redis.expire(self.key, ttl)


class RedisHash(RedisObject):
    def __setitem__(self, key, value):
        self.redis.hset(self.key, key, self._to_redis(value))

    def __getitem__(self, key):
        return self.get(key)

    def __delitem__(self, key):
        return self.redis.hdel(self.key, key)

    def __contains__(self, key):
        return key in self.keys()

    def get(self, key):
        return self._from_redis(self.redis.hget(self.key, key))

    def keys(self):
        return self.redis.hkeys(self.key)

    def items(self):
        return {k: self._from_redis(v) for k, v in self.redis.hgetall(self.key).iteritems()}

    def values(self):
        return self.items().values()

    def update(self, dct):
        pipe = self.redis.pipeline()
        for k, v in dct.iteritems():
            pipe.hset(self.key, k, self._to_redis(v))
        pipe.execute()


class JSONRedisHash(RedisHash):
    def __init__(self, key, **kwargs):
        kwargs['serializer'] = json
        super(JSONRedisHash, self).__init__(key, **kwargs)


class RedisList(RedisObject):
    def __setitem__(self, index, value):
        self.redis.lset(self.key, index, self._to_redis(value))

    def __getitem__(self, index):
        if isinstance(index, slice):
            if slice.step is not None:
                return [self[i] for i in xrange(*index.indices(len(self)))]
            return [self._from_redis(v) for v in \
                self.redis.lrange(self.key, slice.start or 0, slice.stop or -1)]
        elif isinstance(index, int):
            return self._from_redis(self.redis.lindex(self.key, index))
        else:
            raise TypeError("Invalid argument type.")

    def __len__(self):
        return self.redis.llen(self.key)

    def __iter__(self):
        for value in self.redis.lrange(self.key, 0, -1):
            yield self._from_redis(value)

    def __contains__(self, value):
        return value in list(self)

    def append(self, value):
        self.redis.rpush(self.key, self._to_redis(value))

    def extend(self, lst):
        pipe = self.redis.pipeline()
        for v in lst:
            pipe.rpush(self.key, self._to_redis(v))
        pipe.execute()

    def remove(self, value):
        self.redis.lrem(self.key, self._to_redis(value))


class JSONRedisList(RedisList):
    def __init__(self, key, **kwargs):
        kwargs['serializer'] = json
        super(JSONRedisList, self).__init__(key, **kwargs)


class RedisSet(RedisObject):
    def __iter__(self):
        for value in self.redis.smembers(self.key):
            yield self._from_redis(value)

    def __contains__(self, value):
        return self.redis.ismember(self.key, value)

    def add(self, value):
        self.redis.sadd(self.key, self._to_redis(value))

    def update(self, lst):
        pipe = self.redis.pipeline()
        for v in lst:
            pipe.sadd(self.key, self._to_redis(v))
        pipe.execute()

    def remove(self, value):
        self.redis.srem(self.key, self._to_redis(value))

    def pop(self):
        return self._from_redis(self.redis.spop(self.key))

    def move(self, destination, value):
        if isinstance(destination, RedisSet):
            destination = destination.key
        self.redis.smove(self.key, self.destination, self._to_redis(value))

    def diff(self, *other_keys):
        return self._cmp('sdiff', other_keys)

    def inter(self, *other_keys):
        return self._cmp('sinter', other_keys)

    def union(self, *other_keys):
        return self._cmp('union', other_keys)

    def _cmp(self, op, other_keys):
        keys = []
        for k in other_keys:
            if isinstance(k, RedisSet):
                keys.append(k.key)
            else:
                keys.append(k)
        for value in getattr(self.redis, op)(self.key, *keys):
            yield self._from_redis(value)


class JSONRedisSet(RedisSet):
    def __init__(self, key, **kwargs):
        kwargs['serializer'] = json
        super(JSONRedisSet, self).__init__(key, **kwargs)



from jinja2 import ChoiceLoader
from jinja2.ext import ExprStmtExtension
from jinja_macro_tags import configure_environment as configure_macro_environment, FileLoader
from jinja_layout import LayoutExtension
import os
from ..utils import url_for_static, url_for_same, slugify
from .extensions import *
from helpers import *


def configure_environment(env):
    env.loader = ChoiceLoader([env.loader,
        FileLoader(os.path.join(os.path.dirname(__file__), "layout.html"), "frasco_layout.html"),
        FileLoader(os.path.join(os.path.dirname(__file__), "layout.html"), "layout.html")
    ])

    env.add_extension(ExprStmtExtension)
    env.add_extension(LayoutExtension)
    env.add_extension(FlashMessagesExtension)
    configure_macro_environment(env)

    env.globals.update(real_dict=dict,
                       getattr=getattr,
                       url_for_static=url_for_static,
                       url_for_same=url_for_same,
                       html_tag=html_tag,
                       html_attributes=html_attributes,
                       plural=plural)
    env.filters.update(slugify=slugify,
                       nl2br=nl2br,
                       timeago=timeago)

    env.add_extension(jinja_block_as_fragment_extension("content"))
    env.macros.register_file(os.path.join(os.path.dirname(__file__), "macros.html"), alias="frasco.html")


def register_bootstrap_macros(app):
    app.jinja_env.macros.register_directory(os.path.join(os.path.dirname(__file__), "bootstrap"), prefix="bootstrap")



from setuptools import setup, find_packages


setup(
    name='frasco',
    version='1.3.3',
    url='http://github.com/digicoop/frasco',
    license='MIT',
    author='Maxime Bouroumeau-Fuseau',
    author_email='maxime.bouroumeau@gmail.com',
    description='Set of extensions for Flask to develop SaaS applications',
    packages=find_packages(),
    package_data={
        'frasco': [
            'angular/static/*.js',
            'billing/invoicing/emails/*.html',
            'mail/templates/*.html',
            'mail/templates/layouts/*',
            'templating/*.html',
            'templating/bootstrap/*.html',
            'users/emails/users/*.txt',
            'users/templates/users/*.html'
        ],
    },
    zip_safe=False,
    platforms='any',
    install_requires=[
        'Flask',
        'Flask-SQLAlchemy',
        'Flask-Migrate',
        'Flask-Login',
        'Flask-Bcrypt',
        'Flask-Mail',
        'Flask-Babel',
        'Flask-Assets',
        'Flask-WTF',
        'Flask-CORS',
        'Flask-RQ2',
        'psycopg2-binary',
        'PyYAML',
        'jinja-macro-tags',
        'jinja-layout',
        'python-slugify',
        'ago',
        'simplejson',
        'speaklater',
        'requests',
        'apispec',
        'htmlmin',
        'cssmin',
        'jsmin',
        'boto',
        'goslate',
        'premailer',
        'Markdown',
        'inflection',
        'geoip2',
        'redis',
        'python-socketio',
        'eventlet',
        'authlib',
        'stripe',
        'suds',
        'python-dateutil',
        'pyotp'
    ],
    entry_points='''
        [console_scripts]
        frasco=flask.cli:main
    '''
)
